/**
 * This class is used to generate a julia set of.
 * For a few comments see Mandelbrot.java
 */

import java.awt.Image;
import java.awt.image.*;
import Rect;
import java.awt.Component;
import Fractal;

public class Julia extends Fractal{

    protected int maxiter;
    protected boolean touched;
    private int[] buff;
    private int buff_w, buff_h;
    
    private double c_re = -0.24;
    private double c_im = 0.14;
    
    public Julia() {
        all_bounds = new Rect(-1.5, 1.5, -1.5, 1.5);
        bounds = new Rect(all_bounds);
        maxiter = 25;
        touched = true;
        buff = null;
    }
    
    public boolean needsRegeneration() {
        if (touched)
            return true;
        else
            return false;
    }

    public void setC(double re, double im) {
        c_re = re;
        c_im = im;
        touched = true;
    }
    
    public int[] render(int width, int height, boolean quick) {
        if ((buff == null) || (buff_w != width) || (buff_h != height))
            buff = new int[width*height];
        
        int[] color = new int[3];
        int i;
        double a, b, e;

        for (int yp=0; yp < height; yp++) {
            for (int xp=0; xp < width; xp++) {
                a = ((double)xp/width) * (bounds.get(Rect.XMAX)-bounds.get(Rect.XMIN)) + bounds.get(Rect.XMIN);
                b = (1-(double)yp/height) * (bounds.get(Rect.YMAX)-bounds.get(Rect.YMIN)) + bounds.get(Rect.YMIN);
                          
                for (i=1; i < maxiter; i++) {
                    e = a;
                    a = a*a - b*b;
                    b = 2*e*b;
                    a+=c_re;
                    b+=c_im;
                    if (Math.sqrt(a*a + b*b) > 2)
                        break;
                }
                
                if (i >= maxiter) {
                    color[0] = color[1] = color[2] = 0;
                } else {
                    color[0] = (int)((double)i/maxiter * 255 * 3);
                    color[1] = (int)((double)i/maxiter * 255 * 3/2);
                    color[2] = (int)((double)i/maxiter * 255 * 1);
                    if (color[0] > 255) color[0] = 255;
                    if (color[1] > 255) color[1] = 255;
                    if (color[2] > 255) color[2] = 255; //shouldn't happen
                }
                
                buff[(yp * width) + xp] = (255 << 24) | (color[0] << 16) |
                    (color[1] << 8) | (color[2]);
            }
        }
        touched = false;
        return (buff);
    }
    
    public void setBounds(Rect b) {
        touched = true;
        bounds = new Rect(b);
    }

    public int getIterations() {
        return (maxiter);
    }
    
    public void setIterations(int i) {
        maxiter = i;
    }
    
    public Rect getBounds() {
        return new Rect(bounds);
    }
    
    public Rect getAllBounds() {
        return (new Rect(all_bounds));
    }
    
    public String toString() {
        return ("Julia fractal");
    }
    
    public void setAxisLabels(java.awt.Label x, java.awt.Label y) {
        x.setText("Re z");
        y.setText("Im z");
    }
}
